@@ -6,8 +6,10 @@
 # http://apache.org/licenses/LICENSE-2.0> or the MIT license <LICENSE-MIT or
 # http://opensource.org/licenses/MIT>, at your option. This file may not be
 # copied, modified, or distributed except according to those terms.
+
 import logging
 import os.path
+import sys
 from hashlib import sha256
 from typing import BinaryIO, Callable, List, Optional, Type, TypeVar
 
@@ -21,6 +23,7 @@ from pynitrokey.cli.exceptions import CliException
 from pynitrokey.helpers import (
     DownloadProgressBar,
     Retries,
+    check_experimental_flag,
     local_print,
     require_windows_admin,
 )
@@ -487,7 +490,31 @@ def get_config(ctx: Context, key: str) -> None:
 @click.argument("key")
 @click.argument("value")
 def set_config(ctx: Context, key: str, value: str) -> None:
-    """Query a config value."""
+    """
+    Set a config value.
+
+    This command should not be used directly as it may have unexpected
+    side effects, for example resetting an application.  It is only intended
+    for development and testing.
+    """
+
+    # config fields that don’t have side effects
+    whitelist = [
+        "fido.disable_skip_up_timeout",
+    ]
+
+    if key not in whitelist:
+        print(
+            "Changing configuration values can have unexpected side effects, including data loss.",
+            file=sys.stderr,
+        )
+        print(
+            "This command should only be used for development and testing.",
+            file=sys.stderr,
+        )
+
+        click.confirm("Do you want to continue anyway?", abort=True)
+
     with ctx.connect_device() as device:
         admin = AdminApp(device)
         admin.set_config(key, value)
@@ -503,6 +530,43 @@ def version(ctx: Context) -> None:
         local_print(version)
 
 
+@nk3.command()
+@click.pass_obj
+@click.option(
+    "--experimental",
+    default=False,
+    is_flag=True,
+    help="Allow to execute experimental features",
+    hidden=True,
+)
+def factory_reset(ctx: Context, experimental: bool) -> None:
+    """Factory reset all functionality of the device"""
+    check_experimental_flag(experimental)
+    with ctx.connect_device() as device:
+        device.factory_reset()
+
+
+# We consciously do not allow resetting the admin app
+APPLICATIONS_CHOICE = click.Choice(["fido", "opcard", "secrets", "piv", "webcrypt"])
+
+
+@nk3.command()
+@click.pass_obj
+@click.argument("application", type=APPLICATIONS_CHOICE, required=True)
+@click.option(
+    "--experimental",
+    default=False,
+    is_flag=True,
+    help="Allow to execute experimental features",
+    hidden=True,
+)
+def factory_reset_app(ctx: Context, application: str, experimental: bool) -> None:
+    """Factory reset all functionality of an application"""
+    check_experimental_flag(experimental)
+    with ctx.connect_device() as device:
+        device.factory_reset_app(application)
+
+
 @nk3.command()
 @click.pass_obj
 def wink(ctx: Context) -> None:
